{-

 Copyright (c) 2005 Jesper Louis Andersen <jlouis@mongers.org>

 Permission to use, copy, modify, and distribute this software for any
 purpose with or without fee is hereby granted, provided that the above
 copyright notice and this permission notice appear in all copies.

 THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Conjure.OptionParser
-- Copyright   :  (c) Jesper Louis Andersen, 2005
-- License     :  BSD-style (see the file libraries/base/LICENSE)
-- 
-- Maintainer  :  jlouis@mongers.org
-- Stability   :  believed to be stable
-- Portability :  portable
-- 
-- Utilize 'System.Console.GetOpt' to parse command line options of
-- Conjure.
-----------------------------------------------------------------------------
module Conjure.OptionParser
  (
   Flag(..),
   getOpts
  )
where
    
import System.Exit
import System.Console.GetOpt
import Data.Maybe ( fromMaybe )
import Control.Monad (when)

data Flag 
     = Verbose  | Version | RarestCutoff String | Help
       deriving (Eq,Show)

options :: [OptDescr Flag]
options =
  [ Option ['v']        ["verbose"] (NoArg Verbose) "Provide verbose output",
    Option ['V','?']    ["version"] (NoArg Version) "Show version number",
    Option ['R']        ["rarest-cutoff"] 
                        (OptArg rarest_cut_point "INTEGER")
                        "Cutoff point for rarest first piece retrieval",
    Option []           ["help"]    (NoArg Help)    "Show this help" ]

rarest_cut_point :: Maybe String -> Flag
rarest_cut_point = RarestCutoff . fromMaybe "10"

getOpts :: [String] -> IO ([Flag], [String])
getOpts argv =
  case getOpt Permute options argv of
       (o, files, []  ) -> do when (Help `elem` o) (do putStrLn usage 
                                                       exitWith ExitSuccess)
                              return (o, files)
       (_, _, errs)      -> ioError (userError (concat errs ++ usage))
 where header = "Usage: conjure [OPTION...] file"
       usage  = usageInfo header options
